@@ -9,12 +9,13 @@ my $sometime = DateTime->now;
 
 any '/conditional' => sub {
     http_conditional {
-#       etag            => "x",
+        etag            => "x",
         last_modified   => $sometime,
-        required        => true
+#       required        => true
         } => sub {
         "Condition met to execute the request..."
-    }
+    };
+    "So we fall through now?"
 };
 
 dance;
\ No newline at end of file
